@@ -68,7 +68,9 @@ struct KIOFuseVFS::FuseLLOps : public fuse_lowlevel_ops
 		flush = &KIOFuseVFS::flush;
 		release = &KIOFuseVFS::release;
 		fsync = &KIOFuseVFS::fsync;
+		opendir = &KIOFuseVFS::opendir;
 		readdir = &KIOFuseVFS::readdir;
+		releasedir = &KIOFuseVFS::releasedir;
 	}
 };
 
@@ -296,10 +298,7 @@ void KIOFuseVFS::setUseFileJob(bool useFileJob)
 void KIOFuseVFS::mountUrl(QUrl url, std::function<void (const QString &, int)> callback)
 {
 	// First make sure it actually exists
-	QUrl urlWithoutPassword = url;
-	urlWithoutPassword.setPassword({});
-
-	qDebug(KIOFUSE_LOG) << "Stating" << urlWithoutPassword << "for mount";
+	qDebug(KIOFUSE_LOG) << "Stating" << url.toDisplayString() << "for mount";
 	auto statJob = KIO::stat(url);
 	statJob->setSide(KIO::StatJob::SourceSide); // Be "optimistic" to allow accessing
 	                                            // files over plain HTTP
@@ -335,10 +334,7 @@ QStringList KIOFuseVFS::mapUrlToVfs(QUrl url)
 
 void KIOFuseVFS::findAndCreateOrigin(QUrl url, QStringList pathElements, std::function<void (const QString &, int)> callback)
 {
-	QUrl urlWithoutPassword = url;
-	urlWithoutPassword.setPassword({});
-
-	qDebug(KIOFUSE_LOG) << "Trying origin" << urlWithoutPassword;
+	qDebug(KIOFUSE_LOG) << "Trying origin" << url.toDisplayString();
 	auto statJob = KIO::stat(url);
 	statJob->setSide(KIO::StatJob::SourceSide); // Be "optimistic" to allow accessing
 	                                            // files over plain HTTP
@@ -354,7 +350,7 @@ void KIOFuseVFS::findAndCreateOrigin(QUrl url, QStringList pathElements, std::fu
 			return callback({}, kioErrorToFuseError(statJob->error()));
 		}
 
-		qDebug(KIOFUSE_LOG) << "Origin found at" << urlWithoutPassword;
+		qDebug(KIOFUSE_LOG) << "Origin found at" << url.toDisplayString();
 
 		auto targetPathComponents = mapUrlToVfs(url);
 
@@ -393,6 +389,22 @@ void KIOFuseVFS::findAndCreateOrigin(QUrl url, QStringList pathElements, std::fu
 		if(!finalNode)
 		{
 			finalNode = createNodeFromUDSEntry(statJob->statResult(), currentNode->m_stat.st_ino, targetPathComponents.last());
+			if(!finalNode)
+			{
+				qWarning(KIOFUSE_LOG) << "Unable to create a valid final node for" << url.toDisplayString() << "from its UDS Entry";
+				return callback({}, EIO);
+			}
+
+			// Some ioslaves like man:/ implement "index files" for folders (including /) by making
+			// them look as regular file when stating, but they also support listDir for directory
+			// functionality. This behaviour is not compatible, so just reject it outright.
+			if((url.path().isEmpty() || url.path() == QStringLiteral("/"))
+			   && !S_ISDIR(finalNode->m_stat.st_mode))
+			{
+				qWarning(KIOFUSE_LOG) << "Root of mount at" << url.toDisplayString() << "not a directory";
+				return callback({}, ENOTDIR);
+			}
+
 			insertNode(finalNode);
 		}
 
@@ -1025,9 +1037,8 @@ static void appendDirentry(std::vector<char> &dirbuf, fuse_req_t req, const char
 	fuse_add_direntry(req, dirbuf.data() + oldsize, dirbuf.size() + oldsize, name, stbuf, dirbuf.size());
 }
 
-void KIOFuseVFS::readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off, fuse_file_info *fi)
+void KIOFuseVFS::opendir(fuse_req_t req, fuse_ino_t ino, fuse_file_info *fi)
 {
-	Q_UNUSED(fi);
 	KIOFuseVFS *that = reinterpret_cast<KIOFuseVFS*>(fuse_req_userdata(req));
 	auto node = that->nodeForIno(ino);
 	if(!node)
@@ -1044,15 +1055,18 @@ void KIOFuseVFS::readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
 		return;
 	}
 
-	that->awaitChildrenComplete(dirNode, [=](int error){
+	node->m_openCount += 1;
+
+	that->awaitChildrenComplete(dirNode, [=, myfi=*fi](int error) mutable {
 		if(error)
 		{
 			fuse_reply_err(req, error);
 			return;
 		}
 
-		std::vector<char> dirbuf;
-		appendDirentry(dirbuf, req, ".", &node->m_stat);
+		auto dirbuf = std::make_unique<std::vector<char>>();
+
+		appendDirentry(*dirbuf, req, ".", &node->m_stat);
 
 		std::shared_ptr<KIOFuseNode> parentNode;
 		if(node->m_parentIno != KIOFuseIno::DeletedRoot)
@@ -1060,27 +1074,88 @@ void KIOFuseVFS::readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off,
 		if(!parentNode)
 			parentNode = that->nodeForIno(KIOFuseIno::Root);
 		if(parentNode)
-			appendDirentry(dirbuf, req, "..", &parentNode->m_stat);
+			appendDirentry(*dirbuf, req, "..", &parentNode->m_stat);
 
 		for(auto ino : dirNode->m_childrenInos)
 		{
 			auto child = that->nodeForIno(ino);
 			if(!child)
 			{
-				qWarning(KIOFUSE_LOG) << "Node" << node->m_nodeName << "references nonexistant child";
+				qWarning(KIOFUSE_LOG) << "Node" << node->m_nodeName << "references nonexistent child";
 				continue;
 			}
 
-			appendDirentry(dirbuf, req, qPrintable(child->m_nodeName), &child->m_stat);
+			appendDirentry(*dirbuf, req, qPrintable(child->m_nodeName), &child->m_stat);
 		}
 
-		if(off < off_t(dirbuf.size()))
-			fuse_reply_buf(req, dirbuf.data() + off, std::min(off_t(size), off_t(dirbuf.size()) - off));
-		else
-			fuse_reply_buf(req, nullptr, 0);
+		myfi.fh = reinterpret_cast<uint64_t>(dirbuf.release());
+
+		if (!(myfi.flags & O_NOATIME))
+			clock_gettime(CLOCK_REALTIME, &node->m_stat.st_atim);
+
+		fuse_reply_open(req, &myfi);
 	});
 }
 
+void KIOFuseVFS::readdir(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off, struct fuse_file_info *fi)
+{
+	KIOFuseVFS *that = reinterpret_cast<KIOFuseVFS*>(fuse_req_userdata(req));
+	auto node = that->nodeForIno(ino);
+	if(!node)
+	{
+		fuse_reply_err(req, EIO);
+		return;
+	}
+
+	auto dirNode = std::dynamic_pointer_cast<KIOFuseDirNode>(node);
+	if(!dirNode)
+	{
+		fuse_reply_err(req, ENOTDIR);
+		return;
+	}
+
+	std::vector<char>* dirbuf = reinterpret_cast<std::vector<char>*>(fi->fh);
+	if(!dirbuf)
+	{
+		fuse_reply_err(req, EIO);
+		return;
+	}
+
+	if(off < off_t(dirbuf->size()))
+		fuse_reply_buf(req, dirbuf->data() + off, std::min(off_t(size), off_t(dirbuf->size()) - off));
+	else
+		fuse_reply_buf(req, nullptr, 0);
+}
+
+void KIOFuseVFS::releasedir(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info *fi)
+{
+	KIOFuseVFS *that = reinterpret_cast<KIOFuseVFS*>(fuse_req_userdata(req));
+	auto node = that->nodeForIno(ino);
+	if(!node)
+	{
+		fuse_reply_err(req, EIO);
+		return;
+	}
+
+	auto dirNode = std::dynamic_pointer_cast<KIOFuseDirNode>(node);
+	if(!dirNode)
+	{
+		fuse_reply_err(req, ENOTDIR);
+		return;
+	}
+
+	node->m_openCount -= 1;
+	if(std::vector<char> *ptr = reinterpret_cast<std::vector<char>*>(fi->fh))
+  	{
+		delete ptr;
+  	}
+	else{
+		qWarning(KIOFUSE_LOG) << "File handler of node" << node->m_nodeName << "already null";
+  	}
+	  
+	fuse_reply_err(req, 0);
+}
+
 void KIOFuseVFS::read(fuse_req_t req, fuse_ino_t ino, size_t size, off_t off, fuse_file_info *fi)
 {
 	Q_UNUSED(fi);
@@ -1422,7 +1497,7 @@ std::shared_ptr<KIOFuseNode> KIOFuseVFS::nodeByName(const std::shared_ptr<KIOFus
 		auto child = nodeForIno(ino);
 		if(!child)
 		{
-			qWarning(KIOFUSE_LOG) << "Node" << parent->m_nodeName << "references nonexistant child";
+			qWarning(KIOFUSE_LOG) << "Node" << parent->m_nodeName << "references nonexistent child";
 			continue;
 		}
 
@@ -1731,12 +1806,12 @@ std::shared_ptr<KIOFuseNode> KIOFuseVFS::createNodeFromUDSEntry(const KIO::UDSEn
 			attr.st_gid = gr->gr_gid;
 	}
 
-	if(entry.contains(KIO::UDSEntry::UDS_LOCAL_PATH) || entry.contains(KIO::UDSEntry::UDS_URL))
+	if(entry.contains(KIO::UDSEntry::UDS_LOCAL_PATH) || entry.contains(KIO::UDSEntry::UDS_TARGET_URL))
 	{
 		// Create as symlink if possible
 		QString target = entry.stringValue(KIO::UDSEntry::UDS_LOCAL_PATH);
 		if(target.isEmpty())
-			target = QUrl(entry.stringValue(KIO::UDSEntry::UDS_URL)).toLocalFile();
+			target = QUrl(entry.stringValue(KIO::UDSEntry::UDS_TARGET_URL)).toLocalFile();
 
 		if(!target.isEmpty())
 		{
@@ -1755,7 +1830,9 @@ std::shared_ptr<KIOFuseNode> KIOFuseVFS::createNodeFromUDSEntry(const KIO::UDSEn
 		{
 			attr.st_mode |= S_IFREG;
 			std::shared_ptr<KIOFuseRemoteFileNode> ret = nullptr;
-			const QUrl nodeUrl = QUrl{entry.stringValue(KIO::UDSEntry::UDS_URL)};
+			const QUrl nodeUrl = QUrl{entry.stringValue(KIO::UDSEntry::UDS_TARGET_URL)};
+			if(nodeUrl.isEmpty())
+				return nullptr;
 			if(m_useFileJob && KProtocolManager::supportsOpening(nodeUrl) && KProtocolManager::supportsTruncating(nodeUrl))
 				ret = std::make_shared<KIOFuseRemoteFileJobBasedFileNode>(parentIno, name, attr);
 			else
@@ -2238,10 +2315,7 @@ void KIOFuseVFS::awaitChildMounted(const std::shared_ptr<KIOFuseRemoteDirNode> &
 			return callback({}, ENOENT);
 	}
 
-	QUrl urlWithoutPassword = url;
-	urlWithoutPassword.setPassword({});
-
-	qDebug(KIOFUSE_LOG) << "Mounting" << urlWithoutPassword;
+	qDebug(KIOFUSE_LOG) << "Mounting" << url.toDisplayString();
 	auto statJob = KIO::stat(url);
 	statJob->setSide(KIO::StatJob::SourceSide); // Be "optimistic" to allow accessing
 	                                            // files over plain HTTP
