@@ -8,6 +8,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
+#include <dirent.h>
 
 #include <QProcess>
 #include <QStandardPaths>
@@ -17,6 +18,9 @@
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusReply>
 #include <QDebug>
+
+#include <KProtocolInfo>
+
 #include "kiofuse_interface.h"
 #include "kiofuseprivate_interface.h"
 
@@ -32,10 +36,12 @@ private Q_SLOTS:
 	void testLocalPathToRemoteUrl();
 	void testLocalFileOps();
 	void testLocalDirOps();
+	void testReaddirOps();
 	void testCreationOps();
 	void testRenameOps();
 	void testDeletionOps();
 	void testArchiveOps();
+	void testManWorkaround();
 	void testKioErrorMapping();
 	void testRootLookup();
 	void testFilenameEscaping();
@@ -394,6 +400,63 @@ void FileOpsTest::testLocalDirOps()
 	QCOMPARE(mirrorEntryList, sourceEntryList);
 }
 
+void FileOpsTest::testReaddirOps()
+{
+	QTemporaryDir localDir;
+	QVERIFY(localDir.isValid());
+
+	// Mount the temporary dir
+	QString testDirPath = m_kiofuse_iface.mountUrl(QStringLiteral("file://%1").arg(localDir.path())).value();
+	QVERIFY(!testDirPath.isEmpty());
+
+	// Fill the directory with some files
+	for(unsigned int i=0; i<=10; i++)
+	{
+		QVERIFY(QFile(QStringLiteral("%1/tmpFile%2").arg(testDirPath).arg(i)).open(QIODevice::WriteOnly));
+	}
+
+	DIR *testDir = opendir(qPrintable(testDirPath));
+	QVERIFY(testDir);
+
+	auto opendirCleanup = qScopeGuard([&](){ closedir(testDir); });
+
+	QStringList testDirEntryList;
+	QStringList testDirEntryListUpdated;
+
+	// Get the initial entry list to compare with
+	struct dirent *pDirent = nullptr; 
+	while((pDirent = readdir(testDir)) != nullptr)
+		testDirEntryList.push_back(QString::fromUtf8(pDirent->d_name));
+	testDirEntryList.sort();
+	
+	// Verify that entries remain same even if we add new or remove existing entries
+	QVERIFY(QFile::remove(testDirPath + QStringLiteral("/tmpFile1")));
+	QVERIFY(QFile(testDirPath + QStringLiteral("/addCaseFile")).open(QIODevice::WriteOnly));
+
+	rewinddir(testDir);
+	while((pDirent = readdir(testDir)) != nullptr)
+		testDirEntryListUpdated.push_back(QString::fromUtf8(pDirent->d_name));
+	testDirEntryListUpdated.sort();
+
+	QCOMPARE(testDirEntryListUpdated, testDirEntryList);
+
+	// Verify that entries remain same even if entries are modified while iterating
+	testDirEntryListUpdated.clear();
+	rewinddir(testDir);
+
+	unsigned int count = 1;
+	while((pDirent = readdir(testDir)) != nullptr)
+	{
+		QVERIFY(QFile(QStringLiteral("%1/iterCaseFile%2").arg(testDirPath).arg(count)).open(QIODevice::WriteOnly));
+			
+		testDirEntryListUpdated.push_back(QString::fromUtf8(pDirent->d_name));
+		count++;
+	}
+	testDirEntryListUpdated.sort();
+
+	QCOMPARE(testDirEntryListUpdated, testDirEntryList);
+}
+
 void FileOpsTest::testCreationOps()
 {
 	QTemporaryDir localDir;
@@ -583,6 +646,9 @@ void FileOpsTest::testDeletionOps()
 
 void FileOpsTest::testArchiveOps()
 {
+	if (!KProtocolInfo::isKnownProtocol(QStringLiteral("tar")))
+		QSKIP("Test requires tar protocol to be supported. See README for packages required.");
+
 	QString outerpath = QFINDTESTDATA(QStringLiteral("data/outerarchive.tar.gz"));
 
 	// Mount a file inside the archive
@@ -615,6 +681,25 @@ void FileOpsTest::testArchiveOps()
 	innerfile.close();
 }
 
+void FileOpsTest::testManWorkaround()
+{
+	// The man ioslave has "hybrid" directories which stat as regular files but also support
+	// listDir. This behaviour is not supported and mounting has to fail.
+
+	if (!KProtocolInfo::isKnownProtocol(QStringLiteral("man")))
+		QSKIP("Test requires man protocol to be supported. See README for packages required.");
+
+	QDBusPendingReply<QString> reply = m_kiofuse_iface.mountUrl(QStringLiteral("man:foo"));
+	reply.waitForFinished();
+	QVERIFY(reply.isError());
+	QCOMPARE(reply.error().name(), QStringLiteral("org.kde.KIOFuse.VFS.Error.CannotMount"));
+
+	reply = m_kiofuse_iface.mountUrl(QStringLiteral("man:/"));
+	reply.waitForFinished();
+	QVERIFY(reply.isError());
+	QCOMPARE(reply.error().name(), QStringLiteral("org.kde.KIOFuse.VFS.Error.CannotMount"));
+}
+
 void FileOpsTest::testKioErrorMapping()
 {
 	QTemporaryFile localFile;
@@ -880,6 +965,9 @@ void FileOpsTest::testSymlinkRewrite()
 	QCOMPARE(readlink(localDir.filePath(QStringLiteral("symlink"))),
 	         localDir.filePath(QStringLiteral("somedir/../somefile")));
 
+	if (!KProtocolInfo::isKnownProtocol(QStringLiteral("tar")))
+		QSKIP("Test requires tar protocol to be supported. See README for packages required.");
+
 	// Mount something with a different origin
 	QString outerpath = QFINDTESTDATA(QStringLiteral("data/outerarchive.tar.gz"));
 	reply = m_kiofuse_iface.mountUrl(QStringLiteral("tar://%1/outerarchive/").arg(outerpath)).value();
